searchData={"items":[{"type":"module","title":"ThousandIsland","doc":"Thousand Island is a modern, pure Elixir socket server, inspired heavily by\n[ranch](https://github.com/ninenines/ranch). It aims to be easy to understand\n& reason about, while also being at least as stable and performant as alternatives.\n\nThousand Island is implemented as a supervision tree which is intended to be hosted\ninside a host application, often as a dependency embedded within a higher-level\nprotocol library such as [Bandit](https://github.com/mtrudel/bandit). Aside from\nsupervising the Thousand Island process tree, applications interact with Thousand\nIsland primarily via the `ThousandIsland.Handler` behaviour.","ref":"ThousandIsland.html"},{"type":"module","title":"Handlers - ThousandIsland","doc":"The `ThousandIsland.Handler` behaviour defines the interface that Thousand Island\nuses to pass `ThousandIsland.Socket`s up to the application level; together they\nform the primary interface that most applications will have with Thousand Island.\nThousand Island comes with a few simple protocol handlers to serve as examples;\nthese can be found in the [examples](https://github.com/mtrudel/thousand_island/tree/main/examples)\nfolder of this project. A simple implementation would look like this:\n\n```elixir\ndefmodule Echo do\n  use ThousandIsland.Handler\n\n  @impl ThousandIsland.Handler\n  def handle_data(data, socket, state) do\n    ThousandIsland.Socket.send(socket, data)\n    {:continue, state}\n  end\nend\n\n{:ok, pid} = ThousandIsland.start_link(port: 1234, handler_module: Echo)\n```\n\nFor more information, please consult the `ThousandIsland.Handler` documentation.","ref":"ThousandIsland.html#module-handlers"},{"type":"module","title":"Starting a Thousand Island Server - ThousandIsland","doc":"A typical use of `ThousandIsland` might look like the following:\n\n```elixir\ndefmodule MyApp.Supervisor do\n  # ... other Supervisor boilerplate\n\n  def init(config) do\n    children = [\n      # ... other children as dictated by your app\n      {ThousandIsland, port: 1234, handler_module: MyApp.ConnectionHandler}\n    ]\n\n    Supervisor.init(children, strategy: :one_for_one)\n  end\nend\n```\n\nYou can also start servers directly via the `start_link/1` function:\n\n```elixir\n{:ok, pid} = ThousandIsland.start_link(port: 1234, handler_module: MyApp.ConnectionHandler)\n```","ref":"ThousandIsland.html#module-starting-a-thousand-island-server"},{"type":"module","title":"Configuration - ThousandIsland","doc":"A number of options are defined when starting a server. The complete list is\ndefined by the `t:ThousandIsland.options/0` type.","ref":"ThousandIsland.html#module-configuration"},{"type":"module","title":"Connection Draining & Shutdown - ThousandIsland","doc":"`ThousandIsland` instances are just a process tree consisting of standard\n`Supervisor`, `GenServer` and `Task` modules, and so the usual rules regarding\nshutdown and shutdown timeouts apply. Immediately upon beginning the shutdown\nsequence the ThousandIsland.ShutdownListener process will cause the listening\nsocket to shut down, which in turn will cause all of the\nThousandIsland.Acceptor processes to shut down as well. At this point all that\nis left in the supervision tree are several layers of Supervisors and whatever\n`Handler` processes were in progress when shutdown was initiated. At this\npoint, standard `Supervisor` shutdown timeout semantics give existing\nconnections a chance to finish things up. `Handler` processes trap exit, so\nthey continue running beyond shutdown until they either complete or are\n`:brutal_kill`ed after their shutdown timeout expires.","ref":"ThousandIsland.html#module-connection-draining-shutdown"},{"type":"module","title":"Logging & Telemetry - ThousandIsland","doc":"As a low-level library, Thousand Island purposely does not do any inline\nlogging of any kind. The `ThousandIsland.Logger` module defines a number of\nfunctions to aid in tracing connections at various log levels, and such logging\ncan be dynamically enabled and disabled against an already running server. This\nlogging is backed by telemetry events internally.\n\nThousand Island emits a rich set of telemetry events including spans for each\nserver, acceptor process, and individual client connection. These telemetry\nevents are documented in the `ThousandIsland.Telemetry` module.","ref":"ThousandIsland.html#module-logging-telemetry"},{"type":"function","title":"ThousandIsland.connection_pids/1","doc":"Gets a list of active connection processes. This is inherently a bit of a leaky notion in the\nface of concurrency, as there may be connections coming and going during the period that this\nfunction takes to run. Callers should account for the possibility that new connections may have\nbeen made since / during this call, and that processes returned by this call may have since\ncompleted. The order that connection processes are returned in is not specified","ref":"ThousandIsland.html#connection_pids/1"},{"type":"function","title":"ThousandIsland.listener_info/1","doc":"Returns information about the address and port that the server is listening on","ref":"ThousandIsland.html#listener_info/1"},{"type":"function","title":"ThousandIsland.resume/1","doc":"Resume a suspended server. This will reopen the listening port, and resume the acceptance of new\nconnections","ref":"ThousandIsland.html#resume/1"},{"type":"function","title":"ThousandIsland.start_link/1","doc":"Starts a `ThousandIsland` instance with the given options. Returns a pid\nthat can be used to further manipulate the server via other functions defined on\nthis module in the case of success, or an error tuple describing the reason the\nserver was unable to start in the case of failure.","ref":"ThousandIsland.html#start_link/1"},{"type":"function","title":"ThousandIsland.stop/2","doc":"Synchronously stops the given server, waiting up to the given number of milliseconds\nfor existing connections to finish up. Immediately upon calling this function,\nthe server stops listening for new connections, and then proceeds to wait until\neither all existing connections have completed or the specified timeout has\nelapsed.","ref":"ThousandIsland.html#stop/2"},{"type":"function","title":"ThousandIsland.suspend/1","doc":"Suspend the server. This will close the listening port, and will stop the acceptance of new\nconnections. Existing connections will stay connected and will continue to be processed.\n\nThe server can later be resumed by calling `resume/1`, or shut down via standard supervision\npatterns.\n\nIf this function returns `:error`, it is unlikely that the server is in a useable state\n\nNote that if you do not explicitly set a port (or if you set port to `0`), then the server will\nbind to a different port when you resume it. This new port can be obtained as usual via the\n`listener_info/1` function. This is not a concern if you explicitly set a port value when first\ninstantiating the server","ref":"ThousandIsland.html#suspend/1"},{"type":"type","title":"ThousandIsland.options/0","doc":"Possible options to configure a server. Valid option values are as follows:\n\n* `handler_module`: The name of the module used to handle connections to this server.\nThe module is expected to implement the `ThousandIsland.Handler` behaviour. Required\n* `handler_options`: A term which is passed as the initial state value to\n`c:ThousandIsland.Handler.handle_connection/2` calls. Optional, defaulting to nil.\n* `genserver_options`: A term which is passed as the value to the handler module's\nunderlying `GenServer.start_link/3` call. Optional, defaulting to []\n* `port`: The TCP port number to listen on. If not specified this defaults to 4000.\nIf a port number of `0` is given, the server will dynamically assign a port number\nwhich can then be obtained via `ThousandIsland.listener_info/1` or\n`ThousandIsland.Socket.sockname/1`\n* `transport_module`: The name of the module which provides basic socket functions.\nThousand Island provides `ThousandIsland.Transports.TCP` and `ThousandIsland.Transports.SSL`,\nwhich provide clear and TLS encrypted TCP sockets respectively. If not specified this\ndefaults to `ThousandIsland.Transports.TCP`\n* `transport_options`: A keyword list of options to be passed to the transport module's\n`c:ThousandIsland.Transport.listen/2` function. Valid values depend on the transport\nmodule specified in `transport_module` and can be found in the documentation for the\n`ThousandIsland.Transports.TCP` and `ThousandIsland.Transports.SSL` modules. Any options\nin terms of interfaces to listen to / certificates and keys to use for SSL connections\nwill be passed in via this option\n* `num_acceptors`: The number of acceptor processes to run. Defaults to 100\n* `num_connections`: The maximum number of concurrent connections which each acceptor will\naccept before throttling connections. Connections will be throttled by having the acceptor\nprocess wait `max_connections_retry_wait` milliseconds, up to `max_connections_retry_count`\ntimes for existing connections to terminate & make room for this new connection. If there is\nstill no room for this new connection after this interval, the acceptor will close the client\nconnection and emit a `[:thousand_island, :acceptor, :spawn_error]` telemetry event. This number\nis expressed per-acceptor, so the total number of maximum connections for a Thousand Island\nserver is `num_acceptors * num_connections`. Defaults to `16_384`\n* `max_connections_retry_wait`: How long to wait during each iteration as described in\n`num_connectors` above, in milliseconds. Defaults to `1000`\n* `max_connections_retry_count`: How many iterations to wait as described in `num_connectors`\nabove. Defaults to `5`\n* `read_timeout`: How long to wait for client data before closing the connection, in\nmilliseconds. Defaults to 60_000\n* `shutdown_timeout`: How long to wait for existing client connections to complete before\nforcibly shutting those connections down at server shutdown time, in milliseconds. Defaults to\n15_000. May also be `:infinity` or `:brutal_kill` as described in the `Supervisor`\ndocumentation\n* `silent_terminate_on_error`: Whether to silently ignore errors returned by the handler or to\nsurface them to the runtime via an abnormal termination result. This only applies to errors\nreturned via `{:error, reason, state}` responses; exceptions raised within a handler are always\nlogged regardless of this value. Note also that telemetry events will always be sent for errors\nregardless of this value. Defaults to false","ref":"ThousandIsland.html#t:options/0"},{"type":"type","title":"ThousandIsland.transport_module/0","doc":"A module implementing `ThousandIsland.Transport` behaviour","ref":"ThousandIsland.html#t:transport_module/0"},{"type":"type","title":"ThousandIsland.transport_options/0","doc":"A keyword list of options to be passed to the transport module's `listen/2` function","ref":"ThousandIsland.html#t:transport_options/0"},{"type":"behaviour","title":"ThousandIsland.Handler","doc":"`ThousandIsland.Handler` defines the behaviour required of the application layer of a Thousand Island server. When starting a\nThousand Island server, you must pass the name of a module implementing this behaviour as the `handler_module` parameter.\nThousand Island will then use the specified module to handle each connection that is made to the server.\n\nThe lifecycle of a Handler instance is as follows:\n\n1. After a client connection to a Thousand Island server is made, Thousand Island will complete the initial setup of the\nconnection (performing a TLS handshake, for example), and then call `c:handle_connection/2`.\n\n2. A handler implementation may choose to process a client connection within the `c:handle_connection/2` callback by\ncalling functions against the passed `ThousandIsland.Socket`. In many cases, this may be all that may be required of\nan implementation & the value `{:close, state}` can be returned which will cause Thousand Island to close the connection\nto the client.\n\n3. In cases where the server wishes to keep the connection open and wait for subsequent requests from the client on the\nsame socket, it may elect to return `{:continue, state}`. This will cause Thousand Island to wait for client data\nasynchronously; `c:handle_data/3` will be invoked when the client sends more data.\n\n4. In the meantime, the process which is hosting connection is idle & able to receive messages sent from elsewhere in your\napplication as needed. The implementation included in the `use ThousandIsland.Handler` macro uses a `GenServer` structure,\nso you may implement such behaviour via standard `GenServer` patterns. Note that in these cases that state is provided (and\nmust be returned) in a `{socket, state}` format, where the second tuple is the same state value that is passed to the various `handle_*` callbacks\ndefined on this behaviour. It also critical to maintain the socket's `read_timeout` value by\nensuring the relevant timeout value is returned as your callback's final argument. Both of these\nconcerns are illustrated in the following example:\n\n    ```elixir\n    defmodule ExampleHandler do\n      use ThousandIsland.Handler\n\n      # ...handle_data and other Handler callbacks\n\n      def handle_cast(msg, from, {socket, state}) do\n        # Do whatever you'd like with msg & from\n        {:reply, :ok, {socket, state}, socket.read_timeout}\n      end\n\n      def handle_cast(msg, {socket, state}) do\n        # Do whatever you'd like with msg\n        {:noreply, {socket, state}, socket.read_timeout}\n      end\n\n      def handle_info(msg, {socket, state}) do\n        # Do whatever you'd like with msg\n        {:noreply, {socket, state}, socket.read_timeout}\n      end\n    end\n    ```\n\nIt is fully supported to intermix synchronous `ThousandIsland.Socket.recv` calls with async return values from `c:handle_connection/2`\nand `c:handle_data/3` callbacks.\n\n# Example\n\nA simple example of a Hello World server is as follows:\n\n```elixir\ndefmodule HelloWorld do\n  use ThousandIsland.Handler\n\n  @impl ThousandIsland.Handler\n  def handle_connection(socket, state) do\n    ThousandIsland.Socket.send(socket, \"Hello, World\")\n    {:close, state}\n  end\nend\n```\n\nAnother example of a server that echoes back all data sent to it is as follows:\n\n```elixir\ndefmodule Echo do\n  use ThousandIsland.Handler\n\n  @impl ThousandIsland.Handler\n  def handle_data(data, socket, state) do\n    ThousandIsland.Socket.send(socket, data)\n    {:continue, state}\n  end\nend\n```\n\nNote that in this example there is no `c:handle_connection/2` callback defined. The default implementation of this\ncallback will simply return `{:continue, state}`, which is appropriate for cases where the client is the first\nparty to communicate.\n\nAnother example of a server which can send and receive messages asynchronously is as follows:\n\n```elixir\ndefmodule Messenger do\n  use ThousandIsland.Handler\n\n  @impl ThousandIsland.Handler\n  def handle_data(msg, _socket, state) do\n    IO.puts(msg)\n    {:continue, state}\n  end\n\n  def handle_info({:send, msg}, {socket, state}) do\n    ThousandIsland.Socket.send(socket, msg)\n    {:noreply, {socket, state}, socket.read_timeout}\n  end\nend\n```\n\nNote that in this example we make use of the fact that the handler process is really just a GenServer to send it messages\nwhich are able to make use of the underlying socket. This allows for bidirectional sending and receiving of messages in\nan asynchronous manner.\n\nYou can pass options to the default handler underlying `GenServer` by passing a `genserver_options` key to `ThousandIsland.start_link/1`\ncontaining `t:GenServer.options/0` to be passed to the last argument of `GenServer.start_link/3`.\n\nPlease note that you should not pass the `name` `t:GenServer.option/0`. If you need to register handler processes for\nlater lookup and use, you should perform process registration in `handle_connection/2`, ensuring the handler process is\nregistered only after the underlying connection is established and you have access to the connection socket and metadata\nvia `ThousandIsland.Socket.peername/1`.\n\nFor example, using a custom process registry via `Registry`:\n\n```elixir\n\ndefmodule Messenger do\n  use ThousandIsland.Handler\n\n  @impl ThousandIsland.Handler\n  def handle_connection(socket, state) do\n    {:ok, {ip, port}} = ThousandIsland.Socket.peername(socket)\n    {:ok, _pid} = Registry.register(MessengerRegistry, {state[:my_key], address}, nil)\n    {:continue, state}\n  end\n\n  @impl ThousandIsland.Handler\n  def handle_data(data, socket, state) do\n    ThousandIsland.Socket.send(socket, data)\n    {:continue, state}\n  end\nend\n```\n\nThis example assumes you have started a `Registry` and registered it under the name `MessengerRegistry`.\n\n# When Handler Isn't Enough\n\nThe `use ThousandIsland.Handler` implementation should be flexible enough to power just about any handler, however if\nthis should not be the case for you, there is an escape hatch available. If you require more flexibility than the\n`ThousandIsland.Handler` behaviour provides, you are free to specify any module which implements `start_link/1` as the\n`handler_module` parameter. The process of getting from this new process to a ready-to-use socket is somewhat\ndelicate, however. The steps required are as follows:\n\n1. Thousand Island calls `start_link/1` on the configured `handler_module`, passing in a tuple\nconsisting of the configured handler and genserver opts. This function is expected to return a\nconventional `GenServer.on_start()` style tuple. Note that this newly created process is not\npassed the connection socket immediately.\n2. The socket will be passed to the new process via a message of the form\n`{:thousand_island_ready, socket, server_config, acceptor_span, start_time}`.\n3. Once the process receives the socket, it must call `ThousandIsland.Socket.handshake/1` with the socket as the sole\nargument in order to finalize the setup of the socket.\n4. The socket is now ready to use.\n\nIn addition to this process, there are several other considerations to be aware of:\n\n* The underlying socket is closed automatically when the handler process ends.\n\n* Handler processes should have a restart strategy of `:temporary` to ensure that Thousand Island does not attempt to\nrestart crashed handlers.\n\n* Handler processes should trap exit if possible so that existing connections can be given a chance to cleanly shut\ndown when shutting down a Thousand Island server instance.\n\n* Some of the `:connection` family of telemetry span events are emitted by the\n`ThousandIsland.Handler` implementation. If you use your own implementation in its place it is\nlikely that such spans will not behave as expected.","ref":"ThousandIsland.Handler.html"},{"type":"callback","title":"ThousandIsland.Handler.handle_close/2","doc":"This callback is called when the underlying socket is closed by the remote end; it should perform any cleanup required\nas it is the last callback called before the process backing this connection is terminated. The underlying socket\nhas already been closed by the time this callback is called. The return value is ignored.\n\nThis callback is not called if the connection is explicitly closed via `ThousandIsland.Socket.close/1`, however it\nwill be called in cases where `handle_connection/2` or `handle_data/3` return a `{:close, state}` tuple.","ref":"ThousandIsland.Handler.html#c:handle_close/2"},{"type":"callback","title":"ThousandIsland.Handler.handle_connection/2","doc":"This callback is called shortly after a client connection has been made, immediately after the socket handshake process has\ncompleted. It is called with the server's configured `handler_options` value as initial state. Handlers may choose to\ninteract synchronously with the socket in this callback via calls to various `ThousandIsland.Socket` functions.\n\nThe value returned by this callback causes Thousand Island to proceed in one of several ways:\n\n* Returning `{:close, state}` will cause Thousand Island to close the socket & call the `c:handle_close/2` callback to\nallow final cleanup to be done.\n* Returning `{:continue, state}` will cause Thousand Island to switch the socket to an asynchronous mode. When the\nclient subsequently sends data (or if there is already unread data waiting from the client), Thousand Island will call\n`c:handle_data/3` to allow this data to be processed.\n* Returning `{:continue, state, timeout}` is identical to the previous case with the\naddition of a timeout. If `timeout` milliseconds passes with no data being received or messages\nbeing sent to the process, the socket will be closed and `c:handle_timeout/2` will be called.\nNote that this timeout is not persistent; it applies only to the interval until the next message\nis received. In order to set a persistent timeout for all future messages (essentially\noverwriting the value of `read_timeout` that was set at server startup), a value of\n`{:persistent, timeout}` may be returned.\n* Returning `{:error, reason, state}` will cause Thousand Island to close the socket & call the `c:handle_error/3` callback to\nallow final cleanup to be done.","ref":"ThousandIsland.Handler.html#c:handle_connection/2"},{"type":"callback","title":"ThousandIsland.Handler.handle_data/3","doc":"This callback is called whenever client data is received after `c:handle_connection/2` or `c:handle_data/3` have returned an\n`{:continue, state}` tuple. The data received is passed as the first argument, and handlers may choose to interact\nsynchronously with the socket in this callback via calls to various `ThousandIsland.Socket` functions.\n\nThe value returned by this callback causes Thousand Island to proceed in one of several ways:\n\n* Returning `{:close, state}` will cause Thousand Island to close the socket & call the `c:handle_close/2` callback to\nallow final cleanup to be done.\n* Returning `{:continue, state}` will cause Thousand Island to switch the socket to an asynchronous mode. When the\nclient subsequently sends data (or if there is already unread data waiting from the client), Thousand Island will call\n`c:handle_data/3` to allow this data to be processed.\n* Returning `{:continue, state, timeout}` is identical to the previous case with the\naddition of a timeout. If `timeout` milliseconds passes with no data being received or messages\nbeing sent to the process, the socket will be closed and `c:handle_timeout/2` will be called.\nNote that this timeout is not persistent; it applies only to the interval until the next message\nis received. In order to set a persistent timeout for all future messages (essentially\noverwriting the value of `read_timeout` that was set at server startup), a value of\n`{:persistent, timeout}` may be returned.\n* Returning `{:error, reason, state}` will cause Thousand Island to close the socket & call the `c:handle_error/3` callback to\nallow final cleanup to be done.","ref":"ThousandIsland.Handler.html#c:handle_data/3"},{"type":"callback","title":"ThousandIsland.Handler.handle_error/3","doc":"This callback is called when the underlying socket encounters an error; it should perform any cleanup required\nas it is the last callback called before the process backing this connection is terminated. The underlying socket\nhas already been closed by the time this callback is called. The return value is ignored.\n\nIn addition to socket level errors, this callback is also called in cases where `handle_connection/2` or `handle_data/3`\nreturn a `{:error, reason, state}` tuple, or when connection handshaking (typically TLS\nnegotiation) fails.","ref":"ThousandIsland.Handler.html#c:handle_error/3"},{"type":"callback","title":"ThousandIsland.Handler.handle_shutdown/2","doc":"This callback is called when the server process itself is being shut down; it should perform any cleanup required\nas it is the last callback called before the process backing this connection is terminated. The underlying socket\nhas NOT been closed by the time this callback is called. The return value is ignored.\n\nThis callback is only called when the shutdown reason is `:normal`, and is subject to the same caveats described\nin `c:GenServer.terminate/2`.","ref":"ThousandIsland.Handler.html#c:handle_shutdown/2"},{"type":"callback","title":"ThousandIsland.Handler.handle_timeout/2","doc":"This callback is called when a handler process has gone more than `timeout` ms without receiving\neither remote data or a local message. The value used for `timeout` defaults to the\n`read_timeout` value specified at server startup, and may be overridden on a one-shot or\npersistent basis based on values returned from `c:handle_connection/2` or `c:handle_data/3`\ncalls. Note that it is NOT called on explicit `ThousandIsland.Socket.recv/3` calls as they have\ntheir own timeout semantics. The underlying socket has NOT been closed by the time this callback\nis called. The return value is ignored.","ref":"ThousandIsland.Handler.html#c:handle_timeout/2"},{"type":"type","title":"ThousandIsland.Handler.handler_result/0","doc":"The value returned by `c:handle_connection/2` and `c:handle_data/3`","ref":"ThousandIsland.Handler.html#t:handler_result/0"},{"type":"module","title":"ThousandIsland.Logger","doc":"Logging conveniences for Thousand Island servers\n\nAllows dynamically adding and altering the log level used to trace connections\nwithin a Thousand Island server via the use of telemetry hooks. Should you wish\nto do your own logging or tracking of these events, a complete list of the\ntelemetry events emitted by Thousand Island is described in the module\ndocumentation for `ThousandIsland.Telemetry`.","ref":"ThousandIsland.Logger.html"},{"type":"function","title":"ThousandIsland.Logger.attach_logger/1","doc":"Start logging Thousand Island at the specified log level. Valid values for log\nlevel are `:error`, `:info`, `:debug`, and `:trace`. Enabling a given log\nlevel implicitly enables all higher log levels as well.","ref":"ThousandIsland.Logger.html#attach_logger/1"},{"type":"function","title":"ThousandIsland.Logger.detach_logger/1","doc":"Stop logging Thousand Island at the specified log level. Disabling a given log\nlevel implicitly disables all lower log levels as well.","ref":"ThousandIsland.Logger.html#detach_logger/1"},{"type":"type","title":"ThousandIsland.Logger.log_level/0","doc":"Supported log levels","ref":"ThousandIsland.Logger.html#t:log_level/0"},{"type":"module","title":"ThousandIsland.Socket","doc":"Encapsulates a client connection's underlying socket, providing a facility to\nread, write, and otherwise manipulate a connection from a client.","ref":"ThousandIsland.Socket.html"},{"type":"function","title":"ThousandIsland.Socket.close/1","doc":"Closes the given socket. Note that a socket is automatically closed when the handler\nprocess which owns it terminates","ref":"ThousandIsland.Socket.html#close/1"},{"type":"function","title":"ThousandIsland.Socket.getopts/2","doc":"Gets the given flags on the socket\n\nErrors are usually from :inet.posix(), however, SSL module defines return type as any()","ref":"ThousandIsland.Socket.html#getopts/2"},{"type":"function","title":"ThousandIsland.Socket.getstat/1","doc":"Returns statistics about the connection.","ref":"ThousandIsland.Socket.html#getstat/1"},{"type":"function","title":"ThousandIsland.Socket.handshake/1","doc":"Handshakes the underlying socket if it is required (as in the case of SSL sockets, for example).\n\nThis is normally called internally by `ThousandIsland.Handler` and does not need to be\ncalled by implementations which are based on `ThousandIsland.Handler`","ref":"ThousandIsland.Socket.html#handshake/1"},{"type":"function","title":"ThousandIsland.Socket.negotiated_protocol/1","doc":"Returns information about the protocol negotiated during transport handshaking (if any).","ref":"ThousandIsland.Socket.html#negotiated_protocol/1"},{"type":"function","title":"ThousandIsland.Socket.peercert/1","doc":"Returns information in the form of `t:public_key.der_encoded()` about the peer certificate of the socket.","ref":"ThousandIsland.Socket.html#peercert/1"},{"type":"function","title":"ThousandIsland.Socket.peername/1","doc":"Returns information in the form of `t:ThousandIsland.Transport.socket_info()` about the remote end of the socket.","ref":"ThousandIsland.Socket.html#peername/1"},{"type":"function","title":"ThousandIsland.Socket.recv/3","doc":"Returns available bytes on the given socket. Up to `length` bytes will be\nreturned (0 can be passed in to get the next 'available' bytes, typically the\nnext packet). If insufficient bytes are available, the function can wait `timeout`\nmilliseconds for data to arrive.","ref":"ThousandIsland.Socket.html#recv/3"},{"type":"function","title":"ThousandIsland.Socket.secure?/1","doc":"Returns whether or not this protocol is secure.","ref":"ThousandIsland.Socket.html#secure?/1"},{"type":"function","title":"ThousandIsland.Socket.send/2","doc":"Sends the given data (specified as a binary or an IO list) on the given socket.","ref":"ThousandIsland.Socket.html#send/2"},{"type":"function","title":"ThousandIsland.Socket.sendfile/4","doc":"Sends the contents of the given file based on the provided offset & length","ref":"ThousandIsland.Socket.html#sendfile/4"},{"type":"function","title":"ThousandIsland.Socket.setopts/2","doc":"Sets the given flags on the socket\n\nErrors are usually from :inet.posix(), however, SSL module defines return type as any()","ref":"ThousandIsland.Socket.html#setopts/2"},{"type":"function","title":"ThousandIsland.Socket.shutdown/2","doc":"Shuts down the socket in the given direction.","ref":"ThousandIsland.Socket.html#shutdown/2"},{"type":"function","title":"ThousandIsland.Socket.sockname/1","doc":"Returns information in the form of `t:ThousandIsland.Transport.socket_info()` about the local end of the socket.","ref":"ThousandIsland.Socket.html#sockname/1"},{"type":"function","title":"ThousandIsland.Socket.telemetry_span/1","doc":"Returns the telemetry span representing the lifetime of this socket","ref":"ThousandIsland.Socket.html#telemetry_span/1"},{"type":"type","title":"ThousandIsland.Socket.t/0","doc":"A reference to a socket along with metadata describing how to use it","ref":"ThousandIsland.Socket.html#t:t/0"},{"type":"module","title":"ThousandIsland.Telemetry","doc":"The following telemetry spans are emitted by thousand_island\n\n## `[:thousand_island, :listener, *]`\n\nRepresents a Thousand Island server listening to a port\n\nThis span is started by the following event:\n\n* `[:thousand_island, :listener, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `local_address`: The IP address that the listener is bound to\n    * `local_port`: The port that the listener is bound to\n    * `transport_module`: The transport module in use\n    * `transport_options`: Options passed to the transport module at startup\n\n\nThis span is ended by the following event:\n\n* `[:thousand_island, :listener, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `local_address`: The IP address that the listener is bound to\n    * `local_port`: The port that the listener is bound to\n    * `transport_module`: The transport module in use\n    * `transport_options`: Options passed to the transport module at startup\n\n## `[:thousand_island, :acceptor, *]`\n\nRepresents a Thousand Island acceptor process listening for connections\n\nThis span is started by the following event:\n\n* `[:thousand_island, :acceptor, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:listener` which created this acceptor\n\nThis span is ended by the following event:\n\n* `[:thousand_island, :acceptor, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `connections`: The number of client requests that the acceptor handled\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:listener` which created this acceptor\n    * `error`: The error that caused the span to end, if it ended in error\n\nThe following events may be emitted within this span:\n\n* `[:thousand_island, :acceptor, :spawn_error]`\n\n    Thousand Island was unable to spawn a process to handle a connection. This occurs when too\n    many connections are in progress; you may want to look at increasing the `num_connections`\n    configuration parameter\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n## `[:thousand_island, :connection, *]`\n\nRepresents Thousand Island handling a specific client request\n\nThis span is started by the following event:\n\n* `[:thousand_island, :connection, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:acceptor` span which accepted\n    this connection\n    * `remote_address`: The IP address of the connected client\n    * `remote_port`: The port of the connected client\n\nThis span is ended by the following event:\n\n* `[:thousand_island, :connection, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `send_oct`: The number of octets sent on the connection\n    * `send_cnt`: The number of packets sent on the connection\n    * `recv_oct`: The number of octets received on the connection\n    * `recv_cnt`: The number of packets received on the connection\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:acceptor` span which accepted\n      this connection\n    * `remote_address`: The IP address of the connected client\n    * `remote_port`: The port of the connected client\n    * `error`: The error that caused the span to end, if it ended in error\n\nThe following events may be emitted within this span:\n\n* `[:thousand_island, :connection, :ready]`\n\n    Thousand Island has completed setting up the client connection\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :async_recv]`\n\n    Thousand Island has asynchronously received data from the client\n\n    This event contains the following measurements:\n\n    * `data`: The data received from the client\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :recv]`\n\n    Thousand Island has synchronously received data from the client\n\n    This event contains the following measurements:\n\n    * `data`: The data received from the client\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :recv_error]`\n\n    Thousand Island encountered an error reading data from the client\n\n    This event contains the following measurements:\n\n    * `error`: A description of the error\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :send]`\n\n    Thousand Island has sent data to the client\n\n    This event contains the following measurements:\n\n    * `data`: The data sent to the client\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :send_error]`\n\n    Thousand Island encountered an error sending data to the client\n\n    This event contains the following measurements:\n\n    * `data`: The data that was being sent to the client\n    * `error`: A description of the error\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :sendfile]`\n\n    Thousand Island has sent a file to the client\n\n    This event contains the following measurements:\n\n    * `filename`: The filename containing data sent to the client\n    * `offset`: The offset (in bytes) within the file sending started from\n    * `bytes_written`: The number of bytes written\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :sendfile_error]`\n\n    Thousand Island encountered an error sending a file to the client\n\n    This event contains the following measurements:\n\n    * `filename`: The filename containing data that was being sent to the client\n    * `offset`: The offset (in bytes) within the file where sending started from\n    * `length`: The number of bytes that were attempted to send\n    * `error`: A description of the error\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:thousand_island, :connection, :socket_shutdown]`\n\n    Thousand Island has shutdown the client connection\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `way`: The direction in which the socket was shut down\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span","ref":"ThousandIsland.Telemetry.html"},{"type":"function","title":"ThousandIsland.Telemetry.monotonic_time/0","doc":"","ref":"ThousandIsland.Telemetry.html#monotonic_time/0"},{"type":"type","title":"ThousandIsland.Telemetry.metadata/0","doc":"","ref":"ThousandIsland.Telemetry.html#t:metadata/0"},{"type":"type","title":"ThousandIsland.Telemetry.span_name/0","doc":"","ref":"ThousandIsland.Telemetry.html#t:span_name/0"},{"type":"type","title":"ThousandIsland.Telemetry.t/0","doc":"","ref":"ThousandIsland.Telemetry.html#t:t/0"},{"type":"behaviour","title":"ThousandIsland.Transport","doc":"This module describes the behaviour required for Thousand Island to interact\nwith low-level sockets. It is largely internal to Thousand Island, however users\nare free to implement their own versions of this behaviour backed by whatever\nunderlying transport they choose. Such a module can be used in Thousand Island\nby passing its name as the `transport_module` option when starting up a server,\nas described in `ThousandIsland`.","ref":"ThousandIsland.Transport.html"},{"type":"callback","title":"ThousandIsland.Transport.accept/1","doc":"Wait for a client connection on the given listener socket. This call blocks until\nsuch a connection arrives, or an error occurs (such as the listener socket being\nclosed).","ref":"ThousandIsland.Transport.html#c:accept/1"},{"type":"callback","title":"ThousandIsland.Transport.close/1","doc":"Closes the given socket.","ref":"ThousandIsland.Transport.html#c:close/1"},{"type":"callback","title":"ThousandIsland.Transport.controlling_process/2","doc":"Transfers ownership of the given socket to the given process. This will always\nbe called by the process which currently owns the socket.","ref":"ThousandIsland.Transport.html#c:controlling_process/2"},{"type":"callback","title":"ThousandIsland.Transport.getopts/2","doc":"Gets the given options on the socket.","ref":"ThousandIsland.Transport.html#c:getopts/2"},{"type":"callback","title":"ThousandIsland.Transport.getstat/1","doc":"Returns stats about the connection on the socket.","ref":"ThousandIsland.Transport.html#c:getstat/1"},{"type":"callback","title":"ThousandIsland.Transport.handshake/1","doc":"Performs an initial handshake on a new client connection (such as that done\nwhen negotiating an SSL connection). Transports which do not have such a\nhandshake can simply pass the socket through unchanged.","ref":"ThousandIsland.Transport.html#c:handshake/1"},{"type":"callback","title":"ThousandIsland.Transport.listen/2","doc":"Create and return a listener socket bound to the given port and configured per\nthe provided options.","ref":"ThousandIsland.Transport.html#c:listen/2"},{"type":"callback","title":"ThousandIsland.Transport.negotiated_protocol/1","doc":"Returns the protocol negotiated as part of handshaking. Most typically this is via TLS'\nALPN or NPN extensions. If the underlying transport does not support protocol negotiation\n(or if one was not negotiated), `{:error, :protocol_not_negotiated}` is returned","ref":"ThousandIsland.Transport.html#c:negotiated_protocol/1"},{"type":"callback","title":"ThousandIsland.Transport.peercert/1","doc":"Returns the peer certificate for the given socket in the form of `t:public_key.der_encoded()`.\nIf the socket is not secure, `{:error, :not_secure}` is returned.","ref":"ThousandIsland.Transport.html#c:peercert/1"},{"type":"callback","title":"ThousandIsland.Transport.peername/1","doc":"Returns information in the form of `t:socket_info()` about the remote end of the socket.","ref":"ThousandIsland.Transport.html#c:peername/1"},{"type":"callback","title":"ThousandIsland.Transport.recv/3","doc":"Returns available bytes on the given socket. Up to `num_bytes` bytes will be\nreturned (0 can be passed in to get the next 'available' bytes, typically the\nnext packet). If insufficient bytes are available, the function can wait `timeout`\nmilliseconds for data to arrive.","ref":"ThousandIsland.Transport.html#c:recv/3"},{"type":"callback","title":"ThousandIsland.Transport.secure?/0","doc":"Returns whether or not this protocol is secure.","ref":"ThousandIsland.Transport.html#c:secure?/0"},{"type":"callback","title":"ThousandIsland.Transport.send/2","doc":"Sends the given data (specified as a binary or an IO list) on the given socket.","ref":"ThousandIsland.Transport.html#c:send/2"},{"type":"callback","title":"ThousandIsland.Transport.sendfile/4","doc":"Sends the contents of the given file based on the provided offset & length","ref":"ThousandIsland.Transport.html#c:sendfile/4"},{"type":"callback","title":"ThousandIsland.Transport.setopts/2","doc":"Sets the given options on the socket. Should disallow setting of options which\nare not compatible with Thousand Island","ref":"ThousandIsland.Transport.html#c:setopts/2"},{"type":"callback","title":"ThousandIsland.Transport.shutdown/2","doc":"Shuts down the socket in the given direction.","ref":"ThousandIsland.Transport.html#c:shutdown/2"},{"type":"callback","title":"ThousandIsland.Transport.sockname/1","doc":"Returns information in the form of `t:socket_info()` about the local end of the socket.","ref":"ThousandIsland.Transport.html#c:sockname/1"},{"type":"type","title":"ThousandIsland.Transport.address/0","doc":"A socket address","ref":"ThousandIsland.Transport.html#t:address/0"},{"type":"type","title":"ThousandIsland.Transport.listen_options/0","doc":"A listener socket options","ref":"ThousandIsland.Transport.html#t:listen_options/0"},{"type":"type","title":"ThousandIsland.Transport.listener_socket/0","doc":"A listener socket used to wait for connections","ref":"ThousandIsland.Transport.html#t:listener_socket/0"},{"type":"type","title":"ThousandIsland.Transport.on_accept/0","doc":"The return value from an accept/1 call","ref":"ThousandIsland.Transport.html#t:on_accept/0"},{"type":"type","title":"ThousandIsland.Transport.on_accept_ssl_error/0","doc":"","ref":"ThousandIsland.Transport.html#t:on_accept_ssl_error/0"},{"type":"type","title":"ThousandIsland.Transport.on_accept_tcp_error/0","doc":"","ref":"ThousandIsland.Transport.html#t:on_accept_tcp_error/0"},{"type":"type","title":"ThousandIsland.Transport.on_close/0","doc":"The return value from a close/1 call","ref":"ThousandIsland.Transport.html#t:on_close/0"},{"type":"type","title":"ThousandIsland.Transport.on_getopts/0","doc":"The return value from a getopts/2 call","ref":"ThousandIsland.Transport.html#t:on_getopts/0"},{"type":"type","title":"ThousandIsland.Transport.on_handshake/0","doc":"The return value from a handshake/1 call","ref":"ThousandIsland.Transport.html#t:on_handshake/0"},{"type":"type","title":"ThousandIsland.Transport.on_handshake_ssl_error/0","doc":"","ref":"ThousandIsland.Transport.html#t:on_handshake_ssl_error/0"},{"type":"type","title":"ThousandIsland.Transport.on_negotiated_protocol/0","doc":"The return value from a negotiated_protocol/1 call","ref":"ThousandIsland.Transport.html#t:on_negotiated_protocol/0"},{"type":"type","title":"ThousandIsland.Transport.on_peercert/0","doc":"The return value from a peercert/1 call","ref":"ThousandIsland.Transport.html#t:on_peercert/0"},{"type":"type","title":"ThousandIsland.Transport.on_peername/0","doc":"The return value from a peername/1 call","ref":"ThousandIsland.Transport.html#t:on_peername/0"},{"type":"type","title":"ThousandIsland.Transport.on_recv/0","doc":"The return value from a recv/3 call","ref":"ThousandIsland.Transport.html#t:on_recv/0"},{"type":"type","title":"ThousandIsland.Transport.on_send/0","doc":"The return value from a send/2 call","ref":"ThousandIsland.Transport.html#t:on_send/0"},{"type":"type","title":"ThousandIsland.Transport.on_sendfile/0","doc":"The return value from a sendfile/4 call","ref":"ThousandIsland.Transport.html#t:on_sendfile/0"},{"type":"type","title":"ThousandIsland.Transport.on_setopts/0","doc":"The return value from a setopts/2 call","ref":"ThousandIsland.Transport.html#t:on_setopts/0"},{"type":"type","title":"ThousandIsland.Transport.on_shutdown/0","doc":"The return value from a shutdown/2 call","ref":"ThousandIsland.Transport.html#t:on_shutdown/0"},{"type":"type","title":"ThousandIsland.Transport.on_sockname/0","doc":"The return value from a sockname/1 call","ref":"ThousandIsland.Transport.html#t:on_sockname/0"},{"type":"type","title":"ThousandIsland.Transport.socket/0","doc":"A socket representing a client connection","ref":"ThousandIsland.Transport.html#t:socket/0"},{"type":"type","title":"ThousandIsland.Transport.socket_get_options/0","doc":"Options which can be set on a socket via setopts/2 (or returned from getopts/1)","ref":"ThousandIsland.Transport.html#t:socket_get_options/0"},{"type":"type","title":"ThousandIsland.Transport.socket_info/0","doc":"Information about an endpoint, either remote ('peer') or local","ref":"ThousandIsland.Transport.html#t:socket_info/0"},{"type":"type","title":"ThousandIsland.Transport.socket_set_options/0","doc":"Options which can be set on a socket via setopts/2 (or returned from getopts/1)","ref":"ThousandIsland.Transport.html#t:socket_set_options/0"},{"type":"type","title":"ThousandIsland.Transport.socket_stats/0","doc":"Connection statistics for a given socket","ref":"ThousandIsland.Transport.html#t:socket_stats/0"},{"type":"type","title":"ThousandIsland.Transport.way/0","doc":"The direction in which to shutdown a connection in advance of closing it","ref":"ThousandIsland.Transport.html#t:way/0"},{"type":"module","title":"ThousandIsland.Transports.SSL","doc":"Defines a `ThousandIsland.Transport` implementation based on TCP SSL sockets\nas provided by Erlang's `:ssl` module. For the most part, users of Thousand\nIsland will only ever need to deal with this module via `transport_options`\npassed to `ThousandIsland` at startup time. A complete list of such options\nis defined via the `t::ssl.tls_server_option` type. This list can be somewhat\ndifficult to decipher; by far the most common values to pass to this transport\nare the following:\n\n* `keyfile`: The path to a PEM encoded key to use for SSL\n* `certfile`: The path to a PEM encoded cert to use for SSL\n* `ip`:  The IP to listen on. Can be specified as:\n  * `{1, 2, 3, 4}` for IPv4 addresses\n  * `{1, 2, 3, 4, 5, 6, 7, 8}` for IPv6 addresses\n  * `:loopback` for local loopback\n  * `:any` for all interfaces (ie: `0.0.0.0`)\n  * `{:local, \"/path/to/socket\"}` for a Unix domain socket. If this option is used, the `port`\n    option *must* be set to `0`.\n\nUnless overridden, this module uses the following default options:\n\n```elixir\nbacklog: 1024,\nnodelay: true,\nlinger: {true, 30},\nsend_timeout: 30_000,\nsend_timeout_close: true,\nreuseaddr: true\n```\n\nThe following options are required for the proper operation of Thousand Island\nand cannot be overridden:\n\n```elixir\nmode: :binary,\nactive: false\n```","ref":"ThousandIsland.Transports.SSL.html"},{"type":"type","title":"ThousandIsland.Transports.SSL.listener_socket/0","doc":"","ref":"ThousandIsland.Transports.SSL.html#t:listener_socket/0"},{"type":"type","title":"ThousandIsland.Transports.SSL.options/0","doc":"","ref":"ThousandIsland.Transports.SSL.html#t:options/0"},{"type":"type","title":"ThousandIsland.Transports.SSL.socket/0","doc":"","ref":"ThousandIsland.Transports.SSL.html#t:socket/0"},{"type":"module","title":"ThousandIsland.Transports.TCP","doc":"Defines a `ThousandIsland.Transport` implementation based on clear TCP sockets\nas provided by Erlang's `:gen_tcp` module. For the most part, users of Thousand\nIsland will only ever need to deal with this module via `transport_options`\npassed to `ThousandIsland` at startup time. A complete list of such options\nis defined via the `t::gen_tcp.listen_option()` type. This list can be somewhat\ndifficult to decipher; by far the most common value to pass to this transport\nis the following:\n\n* `ip`:  The IP to listen on. Can be specified as:\n  * `{1, 2, 3, 4}` for IPv4 addresses\n  * `{1, 2, 3, 4, 5, 6, 7, 8}` for IPv6 addresses\n  * `:loopback` for local loopback\n  * `:any` for all interfaces (i.e.: `0.0.0.0`)\n  * `{:local, \"/path/to/socket\"}` for a Unix domain socket. If this option is used,\n    the `port` option *must* be set to `0`\n\nUnless overridden, this module uses the following default options:\n\n```elixir\nbacklog: 1024,\nnodelay: true,\nlinger: {true, 30},\nsend_timeout: 30_000,\nsend_timeout_close: true,\nreuseaddr: true\n```\n\nThe following options are required for the proper operation of Thousand Island\nand cannot be overridden:\n\n```elixir\nmode: :binary,\nactive: false\n```","ref":"ThousandIsland.Transports.TCP.html"},{"type":"type","title":"ThousandIsland.Transports.TCP.listener_socket/0","doc":"","ref":"ThousandIsland.Transports.TCP.html#t:listener_socket/0"},{"type":"type","title":"ThousandIsland.Transports.TCP.options/0","doc":"","ref":"ThousandIsland.Transports.TCP.html#t:options/0"},{"type":"type","title":"ThousandIsland.Transports.TCP.socket/0","doc":"","ref":"ThousandIsland.Transports.TCP.html#t:socket/0"}],"content_type":"text/markdown"}